/* Include the controller definition */
#include "footbot_nest.h"
/* Function definitions for XML parsing */
#include <argos3/core/utility/configuration/argos_configuration.h>
/* Function definitions for logging */
#include <argos3/core/utility/logging/argos_log.h>

/****************************************/
/****************************************/

CFootBotNest::CFootBotNest() :
   m_pcRABAct(NULL) {}

/****************************************/
/****************************************/

void CFootBotNest::Init(TConfigurationNode& t_node) {
   /*
    * Get sensor/actuator handles
    *
    * The passed string (ex. "range_and_bearing") corresponds to the
    * XML tag of the device whose handle we want to have. For a list of
    * allowed values, type at the command prompt:
    *
    * $ argos3 -q actuators
    *
    * to have a list of all the possible actuators, or
    *
    * $ argos3 -q sensors
    *
    * to have a list of all the possible sensors.
    *
    * NOTE: ARGoS creates and initializes actuators and sensors
    * internally, on the basis of the lists provided the configuration
    * file at the <controllers><footbot_circle><actuators> and
    * <controllers><footbot_circle><sensors> sections. If you forgot to
    * list a device in the XML and then you request it here, an error
    * occurs.
    */
   m_pcRABAct  = GetActuator<CCI_RangeAndBearingActuator  >("range_and_bearing");
   m_pcRABS            = GetSensor  <CCI_RangeAndBearingSensor         >("range_and_bearing"       );

   /* Determine the type of robot */
   const std::string& name = CCI_Controller::GetId();
   std::string Target_label = "target";
   std::string::size_type t_label = name.find(Target_label);
   if(t_label != std::string::npos)
   {
      m_robot_type = 1;    // is target
      m_gradient   = 255;  // the gradient value
   }
}

/****************************************/
/****************************************/

void CFootBotNest::ControlStep() {
   /* Send the type of robot */
   m_pcRABAct->SetData(0,m_robot_type);
   m_pcRABAct->SetData(1,m_gradient);
   m_pcRABAct->SetData(5,1);
   m_pcRABAct->SetData(6,C_node);
   
   /* Write on robot log the sent value */
   // RLOG << "C_node: " << C_node << std::endl;

   UInt8 temp_c_id = 255;
   const CCI_RangeAndBearingSensor::TReadings& tMsgs = m_pcRABS->GetReadings();
   for(size_t i = 0; i < tMsgs.size(); ++i) 
   {
      if(tMsgs[i].Data[1] == m_gradient + 1)
      {
         // std::cout << "node angle:" << tMsgs[i].HorizontalBearing << "\n";
         temp_c_id = tMsgs[i].Data[2];
         // break;
      }
   }
   if (C_node != temp_c_id)
   {
      C_node = temp_c_id;
      m_pcRABAct->SetData(6,C_node);
   }   
}

/****************************************/
/****************************************/

void CFootBotNest::Reset() {
   m_pcRABAct->ClearData();
   const std::string& str = CCI_Controller::GetId();
   // std::cout<<str.find("r")<<"\n";
   std::string str3 = str.substr(4,str.length());
   // std::cout << str <<" "<< str3 << "\n";

   UInt8 m_id = atoi(str3.c_str());

   m_pcRABAct->SetData(2,m_id);

   m_pcRABAct->SetData(6,C_node);
}

/****************************************/
/****************************************/

/*
 * This statement notifies ARGoS of the existence of the controller.
 * It binds the class passed as first argument to the string passed as
 * second argument.
 * The string is then usable in the configuration file to refer to this
 * controller.
 * When ARGoS reads that string in the configuration file, it knows which
 * controller class to instantiate.
 * See also the configuration files for an example of how this is used.
 */
REGISTER_CONTROLLER(CFootBotNest, "footbot_nest_controller")
