/*
 * AUTHOR: Chris <dongdashu.aa@gmail.com>
 *
 * An example controller for the foot-bot.
 *
 * This controller makes the robots behave as gas particles. The robots
 * go straight until they get close enough to another robot, in which
 * case they turn, loosely simulating an elastic collision. The net effect
 * is that over time the robots diffuse in the environment.
 *
 * The controller uses the proximity sensor to detect obstacles and the
 * wheels to move the robot around.
 *
 * This controller is meant to be used with the XML files:
 */

#ifndef FOOTBOT_PHYSARUM_H
#define FOOTBOT_PHYSARUM_H

/*
 * Include some necessary headers.
 */
/* Definition of the CCI_Controller class. */
#include <argos3/core/control_interface/ci_controller.h>
/* Definition of the differential steering actuator */
#include <argos3/plugins/robots/generic/control_interface/ci_differential_steering_actuator.h>
/* Definition of the foot-bot proximity sensor */
#include <argos3/plugins/robots/foot-bot/control_interface/ci_footbot_proximity_sensor.h>
/* Definition of the LEDs actuator */
#include <argos3/plugins/robots/generic/control_interface/ci_leds_actuator.h>

/* Definition of the foot-bot distance_scanner actuator */
#include <argos3/plugins/robots/foot-bot/control_interface/ci_footbot_distance_scanner_actuator.h>
/* Definition of the foot-bot distance_scanner sensor */
#include <argos3/plugins/robots/foot-bot/control_interface/ci_footbot_distance_scanner_sensor.h>

/* Definition of the foot-bot range and bearing sensor */
#include <argos3/plugins/robots/generic/control_interface/ci_range_and_bearing_sensor.h>
/* Definition of the range and bearing actuator */
#include <argos3/plugins/robots/generic/control_interface/ci_range_and_bearing_actuator.h>




/*
 * All the ARGoS stuff in the 'argos' namespace.
 * With this statement, you save typing argos:: every time.
 */
using namespace argos;

/*
 * A controller is simply an implementation of the CCI_Controller class.
 */
class CFootBotPhysarum : public CCI_Controller {

public:

   /* Class constructor. */
   CFootBotPhysarum();

   /* Class destructor. */
   virtual ~CFootBotPhysarum() {}

   /*
    * This function initializes the controller.
    * The 't_node' variable points to the <parameters> section in the XML
    * file in the <controllers><footbot_diffusion_controller> section.
    */
   virtual void Init(TConfigurationNode& t_node);

   /*
    * This function is called once every time step.
    * The length of the time step is set in the XML file.
    */
   virtual void ControlStep();

   /*
    * This function resets the controller to its state right after the
    * Init().
    * It is called when you press the reset button in the GUI.
    * In this example controller there is no need for resetting anything,
    * so the function could have been omitted. It's here just for
    * completeness.
    */
   virtual void Reset();

   /*
    * Called to cleanup what done by Init() when the experiment finishes.
    * In this example controller there is no need for clean anything up,
    * so the function could have been omitted. It's here just for
    * completeness.
    */
   virtual void Destroy() {}

   void Advoid_obstacles();
   void Get_Distance_Scanner();
   void Explorer_Motion();
   void Node_Motion();
   void Update_gradient();
   void Extension_mechanism(UInt8 sum_size,UInt8 index);
   CVector2 GeneralizedPotential(Real aver_gradient, Real sum_size);
   CVector2 Generalized_Dynamic_Advoid_Obstacles();
   CVector2 repulsion_vector();
   Real Generalized_Guassian(Real x, Real m, Real s);
   void Vector_to_Wheel_Velocity_Noscale(CVector2 direction);
   Real Get_Balance_Factor();
   void Temporary_Motion();
   void Lost_Chain_Motion();
   void Check_Chain();
   void Lost_to_Explorer();

   void Caculate_Envir_Factor();
   void Update_Root_ID();
   void Update_sum_c_node();
   void Node_to_Other_State();
   void Spread_e_f();

   void Init_RABA_Data();

   /*
    * Returns the food data
    */
   // UInt8 GetTypeData();
   inline UInt8 GetTypeData() {
      return m_robot_type;
   }

private:

   /* Pointer to the differential steering actuator */
   CCI_DifferentialSteeringActuator* m_pcWheels;
   /* Pointer to the foot-bot proximity sensor */
   CCI_FootBotProximitySensor* m_pcProximity;

   /* Pointer to the foot-bot distance scanner actuator */
   CCI_FootBotDistanceScannerActuator* m_pcDSA;
   /* Pointer to the foot-bot distance scanner sensor */
   CCI_FootBotDistanceScannerSensor* m_pcDSS;

   /* Pointer to the foot-bot range and bearing sensor */
   CCI_RangeAndBearingSensor* m_pcRABS;
   /* Pointer to the foot-bot range and bearing actuator */
   CCI_RangeAndBearingActuator* m_pcRABA;

   /* Pointer to the LEDs actuator */
   CCI_LEDsActuator* m_pcLEDs;

   /*
    * The following variables are used as parameters for the
    * algorithm. You can set their value in the <parameters> section
    * of the XML configuration file, under the
    * <controllers><footbot_diffusion_controller> section.
    */

   /* Maximum tolerance for the angle between
    * the robot heading direction and
    * the closest obstacle detected. */
   CDegrees m_cAlpha;
   /* Maximum tolerance for the proximity reading between
    * the robot and the closest obstacle.
    * The proximity reading is 0 when nothing is detected
    * and grows exponentially to 1 when the obstacle is
    * touching the robot.
    */
   Real m_fDelta;
   /* Wheel speed. */
   Real m_fWheelVelocity;
   /* Angle tolerance range to go straight.
    * It is set to [-alpha,alpha]. */
   CRange<CRadians> m_cGoStraightAngleRange;

   UInt8 m_robot_type = 2;      // 0 --nest; 1 --target; 2 --explorer; 3 --node; 4 --lost chain; 5 --temporary node

   UInt8 m_node_gradient = 255;

   UInt8 u_length_rc = 200;      // Unit length of robot chain (the distance between each node)

   UInt8 counter = 0;            // The life cycle of temporary node and the counter caculate the chain loss

   UInt32 counter_envir = 0;      // The counter to caculate the time of estimate environment

   UInt8 flag_target = 0;        // The flag to show the target was found?    0 -- Not found;   1 --found;

   UInt8 flag_loss = 1;

   UInt8 flag_c_loss = 1;

   UInt8 flag_explorer = 0;      // Used to determine whether there was a explorer.

   UInt8 P_node = 255;           // Parent_node

   UInt8 C_node = 255;           // Child node

   UInt8 m_id;                   // ID

   UInt8 num_C_node = 0;         // the number of child node

   UInt8 m_root_id;              // the root of robot chain

   UInt8 e_f = 1;                // Environmental assessment factors for node

   /* Current robot state */
   enum EState {
      STATE_EXPLORER = 2,
      STATE_NODE = 3,
      STATE_LOST_CHAIN = 4,
      STATE_TEMPORARY = 5
   };

};

#endif
