/*
 * AUTHOR: Chris <dongdashu.aa@gmail.com>
 *
 * An example controller for the physarum.
 */

#ifndef FOOTBOT_NEST_H
#define FOOTBOT_NEST_H

/*
 * Include some necessary headers.
 */
/* Definition of the CCI_Controller class. */
#include <argos3/core/control_interface/ci_controller.h>
/* Definition of the range-and-bearing actuator */
#include <argos3/plugins/robots/generic/control_interface/ci_range_and_bearing_actuator.h>
/* Definition of the foot-bot range and bearing sensor */
#include <argos3/plugins/robots/generic/control_interface/ci_range_and_bearing_sensor.h>

/*
 * All the ARGoS stuff in the 'argos' namespace.
 * With this statement, you save typing argos:: every time.
 */
using namespace argos;

/*
 * A controller is simply an implementation of the CCI_Controller class.
 */
class CFootBotNest : public CCI_Controller {

public:

   /* Class constructor. */
   CFootBotNest();

   /* Class destructor. */
   virtual ~CFootBotNest() {}

   /*
    * This function initializes the controller.
    * The 't_node' variable points to the <parameters> section in the XML
    * file in the <controllers><footbot_circle_controller> section.
    */
   virtual void Init(TConfigurationNode& t_node);

   /*
    * This function is called once every time step.
    * The length of the time step is set in the XML file.
    */
   virtual void ControlStep();

   /*
    * This function resets the controller to its state right after the
    * Init().
    * It is called when you press the reset button in the GUI.
    */
   virtual void Reset();

   /*
    * Called to cleanup what done by Init() when the experiment finishes.
    * In this example controller there is no need for clean anything up,
    * so the function could have been omitted. It's here just for
    * completeness.
    */
   virtual void Destroy() {}

private:

   /* Pointer to the range-and-bearing actuator */
   CCI_RangeAndBearingActuator* m_pcRABAct;
   /* Pointer to the foot-bot range and bearing sensor */
   CCI_RangeAndBearingSensor* m_pcRABS;

   /* Counter whose value is sent to the quad-rotor */
   // UInt64 m_unCounter;

   /* The type of robot */
   UInt8 m_robot_type = 0;   // 0 --nest; 1 --target; 2 --explorer; 3 --node; 
   UInt8 m_gradient   = 0;   // the gradient value

   UInt8 C_node = 255;
};

#endif
