#include "disaster_lf.h"
#include <argos3/core/wrappers/lua/lua_controller.h>
#include <argos3/core/wrappers/lua/lua_utility.h>

/****************************************/
/****************************************/

CDisasterLF::CDisasterLF() {
}

/****************************************/
/****************************************/

CDisasterLF::~CDisasterLF() {
}

/****************************************/
/****************************************/

void CDisasterLF::Init(TConfigurationNode& t_tree) {
   /* Parse parameters */
   GetNodeAttribute(t_tree, "deployment_area_min_corner", m_cDeploymentAreaMinCorner);
   GetNodeAttribute(t_tree, "deployment_area_max_corner", m_cDeploymentAreaMaxCorner);
   /* Get handle to the victims */
   CSpace::TMapPerType& mapFB = GetSpace().GetEntitiesByType("foot-bot");
   for(CSpace::TMapPerType::iterator it = mapFB.begin();
       it != mapFB.end();
       ++it) {
      CFootBotEntity* pcFB = any_cast<CFootBotEntity*>(it->second);
      if(pcFB->GetId().length() >=6 && pcFB->GetId().substr(0, 6) == "victim") {
         m_vecVictims.push_back(pcFB);
      }
   }
   /* Initialize the rest */
   Reset();
}

/****************************************/
/****************************************/

void CDisasterLF::Reset() {
   m_unVictimsFound = 0;
}

/****************************************/
/****************************************/

void CDisasterLF::Destroy() {
}

/****************************************/
/****************************************/

void CDisasterLF::PreStep() {
}

/****************************************/
/****************************************/

void CDisasterLF::PostStep() {
   CountVictimsFound();
}

/****************************************/
/****************************************/

bool CDisasterLF::IsExperimentFinished() {
   return false;
}

/****************************************/
/****************************************/

CColor CDisasterLF::GetFloorColor(const CVector2& c_position_on_plane) {
   if(c_position_on_plane.GetX() >= m_cDeploymentAreaMinCorner.GetX() &&
      c_position_on_plane.GetX() <= m_cDeploymentAreaMaxCorner.GetX() &&
      c_position_on_plane.GetY() >= m_cDeploymentAreaMinCorner.GetY() &&
      c_position_on_plane.GetY() <= m_cDeploymentAreaMaxCorner.GetY()) {
      return CColor::GRAY50;
   }
   return CColor::WHITE;
}

/****************************************/
/****************************************/

void CDisasterLF::CountVictimsFound() {
   /* Reset counter */
   m_unVictimsFound = 0;
   /* Go through victims */
   for(size_t i = 0; i < m_vecVictims.size(); ++i) {
      /* Get reference to robot controller */
      CLuaController& cContr = dynamic_cast<CLuaController&>(m_vecVictims[i]->GetControllableEntity().GetController());
      /* Get 'victim' table */
      lua_getglobal(cContr.GetLuaState(), "victim");
      /* Table found? */
      if(! lua_isnil(cContr.GetLuaState(), -1)) {
         /* Get the field 'discovered' */
         lua_getfield(cContr.GetLuaState(), -1, "discovered");
         /* Check it */
         if((! lua_isnil(cContr.GetLuaState(), -1)) &&  /* Field found? */
            lua_isboolean(cContr.GetLuaState(), -1) &&  /* Is it a boolean? */
            lua_toboolean(cContr.GetLuaState(), -1)) {  /* Is is true? */
            ++m_unVictimsFound;
         }
         /* Cleanup */
         lua_pop(cContr.GetLuaState(), 2);
      }
   }
}

/****************************************/
/****************************************/

REGISTER_LOOP_FUNCTIONS(CDisasterLF, "disaster_lf");
