#include "disaster_qt.h"
#include <argos3/core/wrappers/lua/lua_controller.h>
#include <argos3/core/wrappers/lua/lua_utility.h>

/****************************************/
/****************************************/

CDisasterQT::CDisasterQT() {
   RegisterUserFunction<CDisasterQT,CFootBotEntity>(&CDisasterQT::DrawFootBot);
}

/****************************************/
/****************************************/

CDisasterQT::~CDisasterQT() {
}

/****************************************/
/****************************************/

void CDisasterQT::DrawFootBot(CFootBotEntity& c_entity) {
   /* Get reference to robot controller */
   CLuaController& cContr = dynamic_cast<CLuaController&>(c_entity.GetControllableEntity().GetController());
   /* Get 'robot' table */
   lua_getglobal(cContr.GetLuaState(), "robot");
   /* Get 'debug' table */
   lua_getfield(cContr.GetLuaState(), -1, "debug");
   if((! lua_isnil(cContr.GetLuaState(), -1)) &&   // 'debug' table found
      lua_istable(cContr.GetLuaState(), -1)) {     // and it is a table
      /* Get 'message' field */
      lua_getfield(cContr.GetLuaState(), -1, "message");
      if((! lua_isnil(cContr.GetLuaState(), -1)) &&              // 'message' field found
         (lua_isstring(cContr.GetLuaState(), -1))) {             // and it is a string
         DrawText(CVector3(0.0f, 0.0f, 0.5f),
                  lua_tostring(cContr.GetLuaState(), -1),
                  CColor::BLACK);
      }
      /* Get 'vectors' table */
      lua_getfield(cContr.GetLuaState(), -2, "vectors");
      if((! lua_isnil(cContr.GetLuaState(), -1)) &&              // 'message' field found
         (lua_istable(cContr.GetLuaState(), -1))) {              // and it is a table
         /* Vector label */
         const char* pchVecLabel;
         /* Vector coords */
         CRay3 cVecCoords(CVector3(0.0f, 0.0f, 0.3f),
                          CVector3(0.0f, 0.0f, 0.3f));
         /* Vector color */
         CColor cVecColor;
         /* First key */
         lua_pushnil(cContr.GetLuaState());
         /* Go through all table elements */
         while(lua_next(cContr.GetLuaState(), -2) != 0) {
            /* Now the key (vector label) is at -2, and the value (vector data) is at -1 */
            pchVecLabel = lua_tostring(cContr.GetLuaState(), -2);
            try {
               /* Parse the vector data */
               if(lua_istable(cContr.GetLuaState(), -1)) {
                  /* Parse coordinates */
                  lua_getfield(cContr.GetLuaState(), -1, "coords");
                  if((! lua_isnil(cContr.GetLuaState(), -1)) &&
                     lua_istable(cContr.GetLuaState(), -1)) {
                     /* Get the x coordinate */
                     lua_getfield(cContr.GetLuaState(), -1, "x");
                     /* Get the y coordinate */
                     lua_getfield(cContr.GetLuaState(), -2, "y");
                     /* Set the coordinate values */
                     cVecCoords.GetEnd().SetX(lua_tonumber(cContr.GetLuaState(), -2));
                     cVecCoords.GetEnd().SetY(lua_tonumber(cContr.GetLuaState(), -1));
                     /* Pop values from the stack */
                     lua_pop(cContr.GetLuaState(), 3);
                  }
                  else {
                     /* Pop values from the stack */
                     lua_pop(cContr.GetLuaState(), 1);
                     throw 2;
                  }
                  /* Parse color */
                  lua_getfield(cContr.GetLuaState(), -1, "color");
                  if((! lua_isnil(cContr.GetLuaState(), -1)) &&
                     lua_isstring(cContr.GetLuaState(), -1)) {
                     /* Set the color */
                     cVecColor.Set(lua_tostring(cContr.GetLuaState(), -1));
                  }
                  else cVecColor = CColor::BLACK;
                  /* Pop values from the stack */
                  lua_pop(cContr.GetLuaState(), 1);
               }
               else throw 1;
            }
            catch(SInt32) {
               LOGERR << "[WARNING] [\""
                      << c_entity.GetId()
                      << "\"] Syntax error for vector \""
                      << pchVecLabel
                      << "\"" << std::endl;
            }

            /* Draw vector */
            DrawRay(CRay3(cVecCoords.GetStart(),
                          cVecCoords.GetEnd()),
                    cVecColor,
                    1.0f);
            DrawText(cVecCoords.GetEnd(),
                     pchVecLabel,
                     cVecColor);
            /* Pop values from the stack */
            lua_pop(cContr.GetLuaState(), 1);
         }
      }
      /* Clean up Lua stack */
      lua_pop(cContr.GetLuaState(), 2);
   }
   /* Clean up Lua stack */
   lua_pop(cContr.GetLuaState(), 2);
}

/****************************************/
/****************************************/

REGISTER_QTOPENGL_USER_FUNCTIONS(CDisasterQT, "disaster_qt");
